import qs from 'qs';
import { nanoid } from 'nanoid';
import sha256 from 'crypto-js/sha256';
import { ApiMethodEnum } from "../typings/enum/common.enum";
import { getValueType } from "./type.util";
import { getHostAndProtocol } from "./common.util";
// 获取待加密的path, 后端实际监听的url，一般是剔除 `${protocol}//${hostname}:${port}/${实例名}`后的内容
var getBackendPath = function getBackendPath(opt) {
  var url = opt.url,
    baseUrl = opt.baseUrl,
    params = opt.params;
  var backendPath = url;
  // 如果有host，则剔除host
  var curHost = getHostAndProtocol();
  if (backendPath.startsWith(curHost)) {
    backendPath = backendPath.replace(curHost, '');
  }
  // 剔除用于转发的实例名
  if (baseUrl) {
    if (typeof baseUrl === 'string') {
      var slicedBaseUrl = baseUrl.startsWith('/') ? baseUrl.slice(1) : baseUrl; // 临时剔除baseUrl中的 /
      var slicedBackendPath = backendPath.startsWith('/') ? backendPath.slice(1) : backendPath; // 临时剔除backendPath中的 /
      if (slicedBackendPath.startsWith(slicedBaseUrl)) {
        backendPath = slicedBackendPath.replace(slicedBaseUrl, '');
      }
    } else {
      // 其他类型认为是RegExp类型
      backendPath = backendPath.replace(baseUrl, '');
    }
  }

  // 保证进行加密的字符串是以 / 开头
  if (!backendPath.startsWith('/')) {
    backendPath = "/".concat(backendPath);
  }
  // 拼接查询内容
  if (params) {
    var queryString = "".concat(qs.stringify(params, {
      arrayFormat: 'repeat',
      encode: false
    })); // 不进行转码
    if (queryString) {
      backendPath += backendPath.indexOf('?') > -1 ? "&".concat(queryString) : "?".concat(queryString);
    }
  }
  // 剔除path转码后的特殊字符（避免前后端转码不一致）
  backendPath = backendPath.replace(/%[0-9A-F]{2}/g, '').replace(/[^a-zA-Z0-9-_.!~*()]/g, '');
  return backendPath;
};
// 获取待加密的序列化后的data
var getDataStr = function getDataStr(opt) {
  var method = opt.method,
    data = opt.data;
  if (method === ApiMethodEnum.GET || !data) {
    return '';
  }
  var dataType = getValueType(data);
  switch (dataType) {
    case 'String':
      {
        // 字符串直接返回
        return data;
      }
    case 'Array':
    case 'Object':
      {
        // 数组和普通对象，进行序列化
        return JSON.stringify(data);
      }
    default:
      {
        // 其他类型不考虑，直接返回空字符串，比如：FormData
        return '';
      }
  }
};
// 生成数字签名对象
// ps: 1. 请先判定是否配置了数字签名 2. 请判定当前path是否在白名单列表中
export var createSignature = function createSignature(opt) {
  var url = opt.url,
    baseUrl = opt.baseUrl,
    params = opt.params,
    _opt$method = opt.method,
    method = _opt$method === void 0 ? ApiMethodEnum.GET : _opt$method,
    data = opt.data,
    token = opt.token,
    salt = opt.salt,
    timeDiff = opt.timeDiff;

  // 获取加密的path
  var backendPath = getBackendPath({
    url: url,
    baseUrl: baseUrl,
    params: params
  });
  // 获取待加密的dataStr
  var dataStr = getDataStr({
    method: method,
    data: data
  });
  var appTimeStamp = new Date().valueOf() + timeDiff;
  var appNonce = nanoid(12);
  var signStr = [appTimeStamp, appNonce, backendPath, dataStr, token, salt].filter(function (item) {
    return !!item;
  }).join('');
  var appSignature = sha256(signStr).toString();
  return {
    'app-timestamp': appTimeStamp,
    'app-nonce': appNonce,
    'app-signature': appSignature
  };
};